@@ -18,6 +18,8 @@ package javax.servlet.http;
 
 import java.io.IOException;
 import java.util.Collection;
+import java.util.Map;
+import java.util.function.Supplier;
 
 import javax.servlet.ServletResponse;
 
@@ -88,9 +90,8 @@ public interface HttpServletResponse extends ServletResponse {
      *
      * @deprecated As of version 2.1, use encodeURL(String url) instead
      */
-    @SuppressWarnings("dep-ann")
-    // Spec API does not use @Deprecated
- String encodeUrl(String url);
+    @Deprecated
+    String encodeUrl(String url);
 
     /**
      * @param url the url to be encoded.
@@ -99,9 +100,8 @@ public interface HttpServletResponse extends ServletResponse {
      *
      * @deprecated As of version 2.1, use encodeRedirectURL(String url) instead
      */
-    @SuppressWarnings("dep-ann")
-    // Spec API does not use @Deprecated
- String encodeRedirectUrl(String url);
+    @Deprecated
+    String encodeRedirectUrl(String url);
 
     /**
      * Sends an error response to the client using the specified status code and clears the output buffer. The server
@@ -133,20 +133,33 @@ public interface HttpServletResponse extends ServletResponse {
     void sendError(int sc) throws IOException;
 
     /**
-     * Sends a temporary redirect response to the client using the specified redirect location URL. This method can
-     * accept relative URLs; the servlet container must convert the relative URL to an absolute URL before sending the
-     * response to the client. If the location is relative without a leading '/' the container interprets it as relative
-     * to the current request URI. If the location is relative with a leading '/' the container interprets it as
-     * relative to the servlet container root.
+     * Sends a redirect response to the client using the specified redirect location URL with the status code
+     * {@link #SC_FOUND} 302 (Found), clears the response buffer and commits the response. The response buffer will be
+     * replaced with a short hypertext note as per RFC 9110.
+     * <p>
+     * This method has no effect if called from an include.
+     * <p>
+     * This method accepts both relative and absolute URLs. Absolute URLs passed to this method are used as provided as
+     * the redirect location URL. Relative URLs are converted to absolute URLs. If converting a relative URL to an
+     * absolute URL then:
+     * <ul>
+     * <li>If the location is relative without a leading '/' the container interprets it as relative to the current
+     * request URI.</li>
+     * <li>If the location is relative with a leading '/' the container interprets it as relative to the servlet
+     * container root.</li>
+     * <li>If the location is relative with two leading '/' the container interprets it as a network-path reference (see
+     * <a href="http://www.ietf.org/rfc/rfc3986.txt"> RFC 3986: Uniform Resource Identifier (URI): Generic Syntax</a>,
+     * section 4.2 &quot;Relative Reference&quot;).</li>
+     * </ul>
      * <p>
      * If the response has already been committed, this method throws an IllegalStateException. After using this method,
      * the response should be considered to be committed and should not be written to.
      *
-     * @param location the redirect location URL
+     * @param location the redirect location URL (may be absolute or relative)
      *
-     * @exception IOException           If an input or output exception occurs
-     * @exception IllegalStateException If the response was committed or if a partial URL is given and cannot be
-     *                                      converted into a valid URL
+     * @exception IOException              If an input or output exception occurs
+     * @exception IllegalArgumentException If a relative URL is given and cannot be converted into an absolute URL
+     * @exception IllegalStateException    If the response was already committed when this method was called
      */
     void sendRedirect(String location) throws IOException;
 
@@ -247,9 +260,8 @@ public interface HttpServletResponse extends ServletResponse {
      *                 <code>setStatus(int)</code>, to send an error with a description use
      *                 <code>sendError(int, String)</code>.
      */
-    @SuppressWarnings("dep-ann")
-    // Spec API does not use @Deprecated
- void setStatus(int sc, String sm);
+    @Deprecated
+    void setStatus(int sc, String sm);
 
     /**
      * Get the HTTP status code for this Response.
@@ -295,6 +307,35 @@ public interface HttpServletResponse extends ServletResponse {
      */
     Collection<String> getHeaderNames();
 
+    /**
+     * Configure the supplier of the trailer headers. The supplier will be called in the scope of the thread that
+     * completes the response. <br>
+     * Trailers that don't meet the requirements of RFC 7230, section 4.1.2 will be ignored. <br>
+     * The default implementation is a NO-OP.
+     *
+     * @param supplier The supplier for the trailer headers
+     *
+     * @throws IllegalStateException if this method is called when the underlying protocol does not support trailer
+     *                                   headers or if using HTTP/1.1 and the response has already been committed
+     *
+     * @since Servlet 4.0
+     */
+    default void setTrailerFields(Supplier<Map<String,String>> supplier) {
+        // NO-OP
+    }
+
+    /**
+     * Obtain the supplier of the trailer headers. <br>
+     * The default implementation returns null.
+     *
+     * @return The supplier for the trailer headers
+     *
+     * @since Servlet 4.0
+     */
+    default Supplier<Map<String,String>> getTrailerFields() {
+        return null;
+    }
+
     /*
      * Server status codes; see RFC 7231.
      */
