@@ -154,15 +154,18 @@ public class RewriteValve extends ValveBase {
 
         InputStream is = null;
 
+        // Process configuration file for this valve
         // Process configuration file for this valve
         if (getContainer() instanceof Context) {
             context = true;
             String webInfResourcePath = "/WEB-INF/" + resourcePath;
             is = ((Context) getContainer()).getServletContext().getResourceAsStream(webInfResourcePath);
-            if (containerLog.isDebugEnabled()) {
-                if (is == null) {
-                    containerLog.debug(sm.getString("rewriteValve.noConfiguration", webInfResourcePath));
-                } else {
+            if (is == null) {
+                if (containerLog.isInfoEnabled()) {
+                    containerLog.info(sm.getString("rewriteValve.noConfiguration", webInfResourcePath));
+                }
+            } else {
+                if (containerLog.isDebugEnabled()) {
                     containerLog.debug(sm.getString("rewriteValve.readConfiguration", webInfResourcePath));
                 }
             }
@@ -171,9 +174,9 @@ public class RewriteValve extends ValveBase {
             try {
                 ConfigurationSource.Resource resource = ConfigFileLoader.getSource().getResource(resourceName);
                 is = resource.getInputStream();
-            } catch (IOException e) {
-                if (containerLog.isDebugEnabled()) {
-                    containerLog.debug(sm.getString("rewriteValve.noConfiguration", resourceName), e);
+            } catch (IOException ioe) {
+                if (containerLog.isInfoEnabled()) {
+                    containerLog.info(sm.getString("rewriteValve.noConfiguration", resourceName), ioe);
                 }
             }
         }
@@ -191,8 +194,8 @@ public class RewriteValve extends ValveBase {
         } finally {
             try {
                 is.close();
-            } catch (IOException e) {
-                containerLog.error(sm.getString("rewriteValve.closeError"), e);
+            } catch (IOException ioe) {
+                containerLog.error(sm.getString("rewriteValve.closeError"), ioe);
             }
         }
 
@@ -253,7 +256,7 @@ public class RewriteValve extends ValveBase {
                     for (RewriteCond condition : conditions) {
                         if (containerLog.isTraceEnabled()) {
                             containerLog.trace("Add condition " + condition.getCondPattern() + " test " +
-                                condition.getTestString() + " to rule with pattern " + rule.getPatternString() +
+                                    condition.getTestString() + " to rule with pattern " + rule.getPatternString() +
                                     " and substitution " + rule.getSubstitutionString() +
                                     (condition.isOrnext() ? " [OR]" : "") + (condition.isNocase() ? " [NC]" : ""));
                         }
@@ -274,8 +277,8 @@ public class RewriteValve extends ValveBase {
                         ((Lifecycle) map).start();
                     }
                 }
-            } catch (IOException e) {
-                containerLog.error(sm.getString("rewriteValve.readError"), e);
+            } catch (IOException ioe) {
+                containerLog.error(sm.getString("rewriteValve.readError"), ioe);
             }
         }
         this.mapsConfiguration = mapsConfiguration;
@@ -320,13 +323,13 @@ public class RewriteValve extends ValveBase {
 
         try {
 
-            Resolver resolver = new ResolverImpl(request);
+            Resolver resolver = new ResolverImpl(request, containerLog);
 
             invoked.set(Boolean.TRUE);
 
             // As long as MB isn't a char sequence or affiliated, this has to be converted to a string
             Charset uriCharset = request.getConnector().getURICharset();
-            String originalQueryStringEncoded = request.getQueryString();
+            String queryStringOriginalEncoded = request.getQueryString();
             MessageBytes urlMB = context ? request.getRequestPathMB() : request.getDecodedRequestURIMB();
             urlMB.toChars();
             CharSequence urlDecoded = urlMB.getCharChunk();
@@ -427,10 +430,10 @@ public class RewriteValve extends ValveBase {
                     StringBuilder urlStringEncoded =
                             new StringBuilder(REWRITE_DEFAULT_ENCODER.encode(urlStringRewriteEncoded, uriCharset));
 
-                    if (!qsd && originalQueryStringEncoded != null && !originalQueryStringEncoded.isEmpty()) {
+                    if (!qsd && queryStringOriginalEncoded != null && !queryStringOriginalEncoded.isEmpty()) {
                         if (rewrittenQueryStringRewriteEncoded == null) {
                             urlStringEncoded.append('?');
-                            urlStringEncoded.append(originalQueryStringEncoded);
+                            urlStringEncoded.append(queryStringOriginalEncoded);
                         } else {
                             if (qsa) {
                                 // if qsa is specified append the query
@@ -438,7 +441,7 @@ public class RewriteValve extends ValveBase {
                                 urlStringEncoded.append(
                                         REWRITE_QUERY_ENCODER.encode(rewrittenQueryStringRewriteEncoded, uriCharset));
                                 urlStringEncoded.append('&');
-                                urlStringEncoded.append(originalQueryStringEncoded);
+                                urlStringEncoded.append(queryStringOriginalEncoded);
                             } else if (index == urlStringEncoded.length() - 1) {
                                 // if the ? is the last character delete it, its only purpose was to
                                 // prevent the rewrite module from appending the query string
@@ -537,7 +540,8 @@ public class RewriteValve extends ValveBase {
                         urlStringRewriteEncoded = urlStringRewriteEncoded.substring(0, queryIndex);
                     }
                     // Parse path parameters from rewrite production and populate request path parameters
-                    urlStringRewriteEncoded = org.apache.catalina.util.RequestUtil.stripPathParams(urlStringRewriteEncoded, request);
+                    urlStringRewriteEncoded =
+                            org.apache.catalina.util.RequestUtil.stripPathParams(urlStringRewriteEncoded, request);
                     // Save the current context path before re-writing starts
                     String contextPath = null;
                     if (context) {
@@ -553,24 +557,31 @@ public class RewriteValve extends ValveBase {
 
                     // Step 3. Complete the 2nd stage to encoding.
                     chunk.append(REWRITE_DEFAULT_ENCODER.encode(urlStringRewriteEncoded, uriCharset));
-                    // Decoded and normalized URI
-                    // Rewriting may have denormalized the URL
-                    urlStringRewriteEncoded = RequestUtil.normalize(urlStringRewriteEncoded);
+                    // Rewriting may have denormalized the URL and added encoded characters
+                    // Decode then normalize
+                    String urlStringRewriteDecoded = URLDecoder.decode(urlStringRewriteEncoded, uriCharset.name());
+                    urlStringRewriteDecoded = RequestUtil.normalize(urlStringRewriteDecoded);
                     request.getCoyoteRequest().decodedURI().setChars(MessageBytes.EMPTY_CHAR_ARRAY, 0, 0);
                     chunk = request.getCoyoteRequest().decodedURI().getCharChunk();
                     if (context) {
                         // This is decoded and normalized
                         chunk.append(request.getServletContext().getContextPath());
                     }
-                    chunk.append(URLDecoder.decode(urlStringRewriteEncoded, uriCharset.name()));
-                    // Set the new Query if there is one
-                    if (queryStringRewriteEncoded != null) {
+                    chunk.append(urlStringRewriteDecoded);
+                    // Set the new Query String
+                    if (queryStringRewriteEncoded == null) {
+                         // No new query string. Therefore the original is retained unless QSD is defined.
+                        if (qsd) {
+                            request.getCoyoteRequest().queryString().setChars(MessageBytes.EMPTY_CHAR_ARRAY, 0, 0);
+                        }
+                    } else {
+                        // New query string. Therefore the original is dropped unless QSA is defined (and QSD is not).
                         request.getCoyoteRequest().queryString().setChars(MessageBytes.EMPTY_CHAR_ARRAY, 0, 0);
                         chunk = request.getCoyoteRequest().queryString().getCharChunk();
                         chunk.append(REWRITE_QUERY_ENCODER.encode(queryStringRewriteEncoded, uriCharset));
-                        if (qsa && originalQueryStringEncoded != null && !originalQueryStringEncoded.isEmpty()) {
+                        if (qsa && queryStringOriginalEncoded != null && !queryStringOriginalEncoded.isEmpty()) {
                             chunk.append('&');
-                            chunk.append(originalQueryStringEncoded);
+                            chunk.append(queryStringOriginalEncoded);
                         }
                     }
                     // Set the new host if it changed
@@ -665,6 +676,10 @@ public class RewriteValve extends ValveBase {
                     while (flagsTokenizer.hasMoreElements()) {
                         parseRuleFlag(line, rule, flagsTokenizer.nextToken());
                     }
+                    // If QSD and QSA are present, QSD always takes precedence
+                    if (rule.isQsdiscard()) {
+                        rule.setQsappend(false);
+                    }
                 }
                 return rule;
             } else if (token.equals("RewriteMap")) {
