@@ -126,7 +126,7 @@ import org.apache.tomcat.util.security.PrivilegedSetTccl;
  *          other thread (OT) unless explicitly marked.
  * </pre>
  */
-public class AsyncStateMachine {
+class AsyncStateMachine {
 
     private static final Log log = LogFactory.getLog(AsyncStateMachine.class);
     private static final StringManager sm = StringManager.getManager(AsyncStateMachine.class);
@@ -158,19 +158,19 @@ public class AsyncStateMachine {
             this.isDispatching = isDispatching;
         }
 
-        public boolean isAsync() {
+        boolean isAsync() {
             return isAsync;
         }
 
-        public boolean isStarted() {
+        boolean isStarted() {
             return isStarted;
         }
 
-        public boolean isDispatching() {
+        boolean isDispatching() {
             return isDispatching;
         }
 
-        public boolean isCompleting() {
+        boolean isCompleting() {
             return isCompleting;
         }
     }
@@ -198,32 +198,32 @@ public class AsyncStateMachine {
     private final AbstractProcessor processor;
 
 
-    public AsyncStateMachine(AbstractProcessor processor) {
+    AsyncStateMachine(AbstractProcessor processor) {
         this.processor = processor;
     }
 
 
-    public boolean isAsync() {
+    boolean isAsync() {
         return state.isAsync();
     }
 
-    public boolean isAsyncDispatching() {
+    boolean isAsyncDispatching() {
         return state.isDispatching();
     }
 
-    public boolean isAsyncStarted() {
+    boolean isAsyncStarted() {
         return state.isStarted();
     }
 
-    public boolean isAsyncTimingOut() {
+    boolean isAsyncTimingOut() {
         return state == AsyncState.TIMING_OUT;
     }
 
-    public boolean isAsyncError() {
+    boolean isAsyncError() {
         return state == AsyncState.ERROR;
     }
 
-    public boolean isCompleting() {
+    boolean isCompleting() {
         return state.isCompleting();
     }
 
@@ -233,7 +233,7 @@ public class AsyncStateMachine {
      * @return The time (as returned by {@link System#currentTimeMillis()}) that this connection last transitioned to
      *             async
      */
-    public long getLastAsyncStart() {
+    long getLastAsyncStart() {
         return lastAsyncStart;
     }
 
@@ -241,7 +241,7 @@ public class AsyncStateMachine {
         return generation.get();
     }
 
-    public synchronized void asyncStart(AsyncContextCallback asyncCtxt) {
+    synchronized void asyncStart(AsyncContextCallback asyncCtxt) {
         if (state == AsyncState.DISPATCHED) {
             generation.incrementAndGet();
             updateState(AsyncState.STARTING);
@@ -255,7 +255,7 @@ public class AsyncStateMachine {
         }
     }
 
-    public synchronized void asyncOperation() {
+    synchronized void asyncOperation() {
         if (state == AsyncState.STARTED) {
             updateState(AsyncState.READ_WRITE_OP);
         } else {
@@ -268,7 +268,7 @@ public class AsyncStateMachine {
      * Async has been processed. Whether or not to enter a long poll depends on current state. For example, as per
      * SRV.2.3.3.3 can now process calls to complete() or dispatch().
      */
-    public synchronized SocketState asyncPostProcess() {
+    synchronized SocketState asyncPostProcess() {
         if (state == AsyncState.COMPLETE_PENDING) {
             clearNonBlockingListeners();
             updateState(AsyncState.COMPLETING);
@@ -304,7 +304,7 @@ public class AsyncStateMachine {
     }
 
 
-    public synchronized boolean asyncComplete() {
+    synchronized boolean asyncComplete() {
         Request request = processor.getRequest();
         if ((request == null || !request.isRequestThread()) &&
                 (state == AsyncState.STARTING || state == AsyncState.READ_WRITE_OP)) {
@@ -347,13 +347,13 @@ public class AsyncStateMachine {
     }
 
 
-    public synchronized boolean asyncTimeout() {
+    synchronized boolean asyncTimeout() {
         if (state == AsyncState.STARTED) {
             updateState(AsyncState.TIMING_OUT);
             return true;
         } else if (state == AsyncState.COMPLETING || state == AsyncState.DISPATCHING ||
                 state == AsyncState.DISPATCHED) {
-            // NOOP - App called complete() or dispatch() between the the
+            // NOOP - App called complete() or dispatch() between the
             // timeout firing and execution reaching this point
             return false;
         } else {
@@ -363,7 +363,7 @@ public class AsyncStateMachine {
     }
 
 
-    public synchronized boolean asyncDispatch() {
+    synchronized boolean asyncDispatch() {
         Request request = processor.getRequest();
         if ((request == null || !request.isRequestThread()) &&
                 (state == AsyncState.STARTING || state == AsyncState.READ_WRITE_OP)) {
@@ -406,7 +406,7 @@ public class AsyncStateMachine {
     }
 
 
-    public synchronized void asyncDispatched() {
+    synchronized void asyncDispatched() {
         if (state == AsyncState.DISPATCHING || state == AsyncState.MUST_DISPATCH) {
             updateState(AsyncState.DISPATCHED);
             asyncCtxt.decrementInProgressAsyncCount();
@@ -417,12 +417,12 @@ public class AsyncStateMachine {
     }
 
 
-    public synchronized boolean asyncError() {
+    synchronized boolean asyncError() {
         Request request = processor.getRequest();
         boolean containerThread = (request != null && request.isRequestThread());
 
-        if (log.isDebugEnabled()) {
-            log.debug(sm.getString("asyncStateMachine.asyncError.start"));
+        if (log.isTraceEnabled()) {
+            log.trace(sm.getString("asyncStateMachine.asyncError.start"));
         }
 
         clearNonBlockingListeners();
@@ -430,8 +430,8 @@ public class AsyncStateMachine {
             updateState(AsyncState.MUST_ERROR);
         } else {
             if (hasProcessedError) {
-                if (log.isDebugEnabled()) {
-                    log.debug(sm.getString("asyncStateMachine.asyncError.skip"));
+                if (log.isTraceEnabled()) {
+                    log.trace(sm.getString("asyncStateMachine.asyncError.skip"));
                 }
                 return false;
             }
@@ -452,7 +452,7 @@ public class AsyncStateMachine {
     }
 
 
-    public synchronized void asyncRun(Runnable runnable) {
+    synchronized void asyncRun(Runnable runnable) {
         if (state == AsyncState.STARTING || state == AsyncState.STARTED || state == AsyncState.READ_WRITE_OP) {
             // Execute the runnable using a container thread from the
             // Connector's thread pool. Use a wrapper to prevent a memory leak
@@ -472,7 +472,7 @@ public class AsyncStateMachine {
                     currentThread.setContextClassLoader(this.getClass().getClassLoader());
                 }
 
-                processor.getExecutor().execute(runnable);
+                processor.execute(runnable);
             } finally {
                 if (Constants.IS_SECURITY_ENABLED) {
                     PrivilegedAction<Void> pa = new PrivilegedSetTccl(currentThread, oldCL);
@@ -484,7 +484,6 @@ public class AsyncStateMachine {
         } else {
             throw new IllegalStateException(sm.getString("asyncStateMachine.invalidAsyncState", "asyncRun()", state));
         }
-
     }
 
 
@@ -498,7 +497,7 @@ public class AsyncStateMachine {
     }
 
 
-    public synchronized void recycle() {
+    synchronized void recycle() {
         // Use lastAsyncStart to determine if this instance has been used since
         // it was last recycled. If it hasn't there is no need to recycle again
         // which saves the relatively expensive call to notifyAll()
@@ -522,8 +521,8 @@ public class AsyncStateMachine {
 
 
     private synchronized void updateState(AsyncState newState) {
-        if (log.isDebugEnabled()) {
-            log.debug(sm.getString("asyncStateMachine.stateChange", state, newState));
+        if (log.isTraceEnabled()) {
+            log.trace(sm.getString("asyncStateMachine.stateChange", state, newState));
         }
         state = newState;
     }
