@@ -29,6 +29,7 @@ import javax.servlet.ReadListener;
 import org.apache.tomcat.util.buf.B2CConverter;
 import org.apache.tomcat.util.buf.MessageBytes;
 import org.apache.tomcat.util.buf.UDecoder;
+import org.apache.tomcat.util.http.Method;
 import org.apache.tomcat.util.http.MimeHeaders;
 import org.apache.tomcat.util.http.Parameters;
 import org.apache.tomcat.util.http.ServerCookies;
@@ -45,15 +46,6 @@ import org.apache.tomcat.util.res.StringManager;
  * <ul>
  * <li>"org.apache.tomcat.request" - allows access to the low-level request object in trusted applications
  * </ul>
- *
- * @author James Duncan Davidson [duncan@eng.sun.com]
- * @author James Todd [gonzo@eng.sun.com]
- * @author Jason Hunter [jch@eng.sun.com]
- * @author Harish Prabandham
- * @author Alex Cruikshank [alex@epitonic.com]
- * @author Hans Bergsten [hans@gefionsoftware.com]
- * @author Costin Manolache
- * @author Remy Maucherat
  */
 public final class Request {
 
@@ -300,10 +292,36 @@ public final class Request {
         return schemeMB;
     }
 
+    /**
+     * Get a MessageBytes instance that holds the current request's HTTP method.
+     *
+     * @return a MessageBytes instance that holds the current request's HTTP method.
+     *
+     * @deprecated Use {@link #getMethod()}, {@link Request#setMethod(String)} and {@link #setMethod(byte[], int, int)}
+     */
+    @Deprecated
     public MessageBytes method() {
         return methodMB;
     }
 
+    public void setMethod(String method) {
+        methodMB.setString(method);
+    }
+
+    public void setMethod(byte[] buf, int start, int len) {
+        String method = Method.bytesToString(buf, start, len);
+        if (method == null) {
+            methodMB.setBytes(buf, start, len);
+            method = methodMB.toStringType();
+        } else {
+            methodMB.setString(method);
+        }
+    }
+
+    public String getMethod() {
+        return methodMB.toStringType();
+    }
+
     public MessageBytes requestURI() {
         return uriMB;
     }
