@@ -81,8 +81,8 @@ public class Cookie {
         // %x21 / %x23-2B / %x2D-3A / %x3C-5B / %x5D-7E (RFC6265)
         // %x80 to %xFF (UTF-8)
         for (int i = 0; i < 256; i++) {
-            isCookieOctet[i] = !(i < 0x21 || i == QUOTE_BYTE || i == COMMA_BYTE ||
-                i == SEMICOLON_BYTE || i == SLASH_BYTE || i == DEL_BYTE);
+            isCookieOctet[i] = !(i < 0x21 || i == QUOTE_BYTE || i == COMMA_BYTE || i == SEMICOLON_BYTE ||
+                    i == SLASH_BYTE || i == DEL_BYTE);
         }
         for (int i = 0; i < 256; i++) {
             isText[i] = !(i < TAB_BYTE || (i > TAB_BYTE && i < SPACE_BYTE) || i == DEL_BYTE);
